@@ -1,4 +1,4 @@
-;;; org-transclusion.el --- transclude text contents of linked target -*- lexical-binding: t; -*-
+;;; org-transclusion.el --- Transclude text content via links -*- lexical-binding: t; -*-
 
 ;; Copyright (C) 2021  Free Software Foundation, Inc.
 
@@ -15,14 +15,14 @@
 ;; You should have received a copy of the GNU General Public License along
 ;; with this program.  If not, see <http://www.gnu.org/licenses/>.
 
-;; Author: Noboru Ota <me@nobiot.com>
-;; Created: 10 Oct 2020
-;; Last modified: 4 December 2021
+;; Author:        Noboru Ota <me@nobiot.com>
+;; Created:       10 October 2020
+;; Last modified: 29 December 2021
 
 ;; URL: https://github.com/nobiot/org-transclusion
 ;; Keywords: org-mode, transclusion, writing
 
-;; Version: 1.0.1
+;; Version: 1.1.0
 ;; Package-Requires: ((emacs "27.1") (org "9.4"))
 
 ;; This file is not part of GNU Emacs.
@@ -53,7 +53,8 @@
   "Insert text contents by way of link references."
   :group 'org
   :prefix "org-transclusion-"
-  :link '(url-link :tag "Github" "https://github.com/nobiot/org-transclusion"))
+  :link '(url-link :tag "Github" "https://github.com/nobiot/org-transclusion")
+  :package-version '("Org-transclusion" . "1.0.0"))
 
 (defun org-transclusion-set-extensions (var value)
   "Set VAR to VALUE and `org-transclusion-load-extensions-maybe'.
@@ -62,13 +63,13 @@ Intended for :set property for `customize'."
   (when (featurep 'org-transclusion)
     (org-transclusion-load-extensions-maybe 'force)))
 
-(defcustom org-transclusion-extensions '(org-transclusion-src-lines)
+ (defcustom org-transclusion-extensions '(org-transclusion-src-lines)
   "Extensions to be loaded with org-transclusion.el."
-  :group 'org-transclusion
-  :set 'org-transclusion-set-extensions
+  :set #'org-transclusion-set-extensions
   :type
   '(set :greedy t
         (const :tag "src-lines: Add :src and :lines for non-Org files" org-transclusion-src-lines)
+        (const :tag "font-lock: Add font-lock for Org-transclusion" org-transclusion-font-lock)
 
         (const :tag "indent-mode: Support org-indent-mode" org-transclusion-indent-mode)
         (repeat :tag "Other packages" :inline t (symbol :tag "Package"))))
@@ -76,37 +77,32 @@ Intended for :set property for `customize'."
 (defcustom org-transclusion-add-all-on-activate t
   "Define whether to add all the transclusions on activation.
 When non-nil, automatically add all on `org-transclusion-activate'."
-  :type 'boolean
-  :group 'org-transclusion)
+  :type 'boolean)
 
 (defcustom org-transclusion-exclude-elements (list 'property-drawer)
   "Define the Org elements that are excluded from transcluded copies.
 It is a list of elements to be filtered out.
 Refer to variable `org-element-all-elements' for names of elements accepted."
-  :type '(repeat symbol)
-  :group 'org-transclusion)
+  :type '(repeat symbol))
 
 (defcustom org-transclusion-include-first-section t
   "Define whether or not transclusion for Org files includes \"first section\".
 If t, the section before the first headline is
 transcluded. Default is t."
-  :type 'boolean
-  :group 'org-transclusion)
+  :type 'boolean)
 
 (defcustom org-transclusion-open-source-display-action-list '(nil . nil)
   "Action list used to open source buffer to display.
 
 See `display-buffer' for example options."
   :type display-buffer--action-custom-type
-  :risky t
-  :group 'org-transclusion)
+  :risky t)
 
 (defcustom org-transclusion-mode-lighter
   " OT"
   "Mode-line indicator for minor-mode variable `org-transclusion-mode'."
   :type '(choice (const :tag "No lighter" "") string)
-  :safe 'stringp
-  :group 'org-transclusion)
+  :safe 'stringp)
 
 ;;;; Faces
 
@@ -114,54 +110,43 @@ See `display-buffer' for example options."
   '((((class color) (min-colors 88) (background light)))
     (((class color) (min-colors 88) (background dark)))
     (t ))
-  "Face for source region's fringe being transcluded in another buffer."
-  :group 'org-transclusion)
-
-(defface org-transclusion-source
+  "Face for source region's fringe being transcluded in another buffer.")
+ (defface org-transclusion-source
   '((((class color) (min-colors 88) (background light))
      :background "#ebf6fa" :extend t)
     (((class color) (min-colors 88) (background dark))
      :background "#041529" :extend t)
     (t
      :foreground "darkgray"))
-  "Face for source region being transcluded in another buffer."
-  :group 'org-transclusion)
-
-(defface org-transclusion-source-edit
+  "Face for source region being transcluded in another buffer.")
+ (defface org-transclusion-source-edit
   '((((class color) (min-colors 88) (background light))
      :background "#fff3da" :extend t)
     (((class color) (min-colors 88) (background dark))
      :background "#221000" :extend t)
     (t
      :background "chocolate4" :extend t))
-  "Face for element in the source being edited by another buffer."
-  :group 'org-transclusion)
-
-(defface org-transclusion-fringe
+  "Face for element in the source being edited by another buffer.")
+ (defface org-transclusion-fringe
   '((((class color) (min-colors 88) (background light)))
     (((class color) (min-colors 88) (background dark)))
     (t ))
-  "Face for transcluded region's fringe in the transcluding buffer."
-  :group 'org-transclusion)
-
-(defface org-transclusion
+  "Face for transcluded region's fringe in the transcluding buffer.")
+ (defface org-transclusion
   '((((class color) (min-colors 88) (background light))
      :background "#ebf6fa" :extend t)
     (((class color) (min-colors 88) (background dark))
      :background "#041529" :extend t)
     (t ))
-  "Face for transcluded region in the transcluding buffer."
-  :group 'org-transclusion)
-
-(defface org-transclusion-edit
+  "Face for transcluded region in the transcluding buffer.")
+ (defface org-transclusion-edit
   '((((class color) (min-colors 88) (background light))
      :background "#ebf6fa" :extend t)
     (((class color) (min-colors 88) (background dark))
      :background "#041529" :extend t)
     (t
      :background "forest green" :extend t))
-  "Face for element in the transcluding buffer in the edit mode."
-  :group 'org-transclusion)
+  "Face for element in the transcluding buffer in the edit mode.")
 
 ;;;; Variables
 
@@ -283,19 +268,12 @@ specific keybindings; namely:
 ;;;; Macro
 ;;;; Definining macros before they are used in the rest of package
 ;;;; Flycheck warns with "macro X defined too late"
-(defmacro org-transclusion-with-silent-modifications (&rest body)
-  "Run BODY silently.
-It's like `with-silent-modifications' but keeps the undo list."
+(defmacro org-transclusion-with-inhibit-read-only (&rest body)
+  "Run BODY with `'inhibit-read-only` t."
   (declare (debug t) (indent 0))
-  (let ((modified (make-symbol "modified")))
-    `(let* ((,modified (buffer-modified-p))
-            (inhibit-read-only t)
-            (inhibit-modification-hooks t))
-       (unwind-protect
-           (progn
-             ,@body)
-         (unless ,modified
-           (restore-buffer-modified-p nil))))))
+  `(let* ((inhibit-read-only t))
+     (progn
+           ,@body)))
 
 ;;;; Commands
 
@@ -324,6 +302,8 @@ and variables."
   (add-hook 'after-save-hook #'org-transclusion-after-save-buffer nil t)
   (add-hook 'kill-buffer-hook #'org-transclusion-before-kill nil t)
   (add-hook 'kill-emacs-hook #'org-transclusion-before-kill nil t)
+  (add-hook 'org-export-before-processing-hook
+            #'org-transclusion-inhibit-read-only nil t)
   (org-transclusion-yank-excluded-properties-set)
   (org-transclusion-load-extensions-maybe))
 
@@ -336,6 +316,8 @@ This function also removes all the transclusions in the current buffer."
   (remove-hook 'after-save-hook #'org-transclusion-after-save-buffer t)
   (remove-hook 'kill-buffer-hook #'org-transclusion-before-kill t)
   (remove-hook 'kill-emacs-hook #'org-transclusion-before-kill t)
+  (remove-hook 'org-export-before-processing-hook
+               #'org-transclusion-inhibit-read-only t)
   (org-transclusion-yank-excluded-properties-remove))
 
 ;;;###autoload
@@ -444,7 +426,7 @@ does not support all the elements.
                   nil))
         (let ((beg (point))
               (end))
-          (org-transclusion-with-silent-modifications
+          (org-transclusion-with-inhibit-read-only
             (when (save-excursion
                     (end-of-line) (insert-char ?\n)
                     (org-transclusion-content-insert
@@ -486,7 +468,11 @@ the rest of the buffer unchanged."
           (unless (or (org-transclusion-within-transclusion-p)
                       (plist-get (org-transclusion-keyword-string-to-plist)
                                  :disable-auto))
-            (org-transclusion-add))))
+            ;; Demoted-errors so that one error does not stop the whole process
+            (with-demoted-errors
+                "Not transcluded. Continue to next: %S"
+              (when (org-transclusion-add)
+                (message (format "Transcluded at point %d, line %d" (point) (org-current-line))))))))
       (goto-char marker)
       (move-marker marker nil) ; point nowhere for GC
       t)))
@@ -520,7 +506,7 @@ When success, return the beginning point of the keyword re-inserted."
           (when (org-transclusion-within-live-sync-p)
             (org-transclusion-live-sync-exit))
           (delete-overlay tc-pair-ov)
-          (org-transclusion-with-silent-modifications
+          (org-transclusion-with-inhibit-read-only
             (save-excursion
               (delete-region beg end)
               (when (> indent 0) (indent-to indent))
@@ -565,9 +551,8 @@ the rest of the buffer unchanged."
       (goto-char (point-min))
       (while (setq match (text-property-search-forward 'org-transclusion-type))
         (goto-char (prop-match-beginning match))
-        (org-transclusion-with-silent-modifications
-          (setq point (org-transclusion-remove))
-          (when point (push point list))))
+        (setq point (org-transclusion-remove))
+        (when point (push point list)))
       (goto-char marker)
       (move-marker marker nil) ; point nowhere for GC
       list)))
@@ -738,6 +723,9 @@ set in `before-save-hook'.  It also move the point back to
           (save-excursion
             (goto-char p)
             (org-transclusion-add)))
+        ;; After save and adding all transclusions, the modified flag should be
+        ;; set to nil
+        (restore-buffer-modified-p nil)
         (when org-transclusion-remember-point
           (goto-char org-transclusion-remember-point))
     (progn
@@ -758,7 +746,7 @@ turned off (removed)."
   ;; --, check is done for `buffer-file-name' to see if there is a file visited
   ;; by the buffer. If a "temp" buffer, there is no file being visited.
   (when (and (org-transclusion-remove-all)
-	     (buffer-file-name))
+             (buffer-file-name))
     (set-buffer-modified-p t)
     (save-buffer)))
 
@@ -1163,11 +1151,14 @@ etc.)."
           (setq obj (org-element-map obj org-element-all-elements
                       #'org-transclusion-content-filter-org-only-contents
                       nil nil '(section) nil)))
-	
-        (list :src-content (org-element-interpret-data obj)
+        ;; For Org content, an extra +1 point seems to be added to the end of
+        ;; each element in the normalized content. This adds an extra empty
+        ;; line, which we would like to remove; hence content and src-end gets
+        ;; -1 here.
+        (list :src-content (substring (org-element-interpret-data obj) 0 -1)
               :src-buf (current-buffer)
               :src-beg (point-min)
-              :src-end (point-max))))))
+              :src-end (1- (point-max)))))))
 
 (defun org-transclusion-content-filter-org-exclude-elements (data)
   "Exclude specific elements from DATA.
@@ -1194,7 +1185,7 @@ is non-nil."
       nil
     data))
 
-;;;;-----------------------------------------------------------------------------
+;;;;---------------------------------------------------------------------------
 ;;;; Functions to support non-Org-mode link types
 
 (defun org-transclusion-content-others-default (link _plist)
@@ -1297,16 +1288,19 @@ Error if transclusion is not allowed.
 Currently the following cases are prevented:
 
 Case 1. Element at point is NOT #+transclude:
+        Element is in a block - e.g. example
 Case 2. #+transclude inside another transclusion"
   (cond
    ;; Case 1. Element at point is NOT #+transclude:
    ((let ((elm (org-element-at-point)))
-       (not (and (string= "keyword" (org-element-type elm))
-		 (string= "TRANSCLUDE" (org-element-property :key elm)))))
-    (user-error "Not at a transclude keyword"))
+      (not (and (string= "keyword" (org-element-type elm))
+                (string= "TRANSCLUDE" (org-element-property :key elm)))))
+    (user-error (format "Not at a transclude keyword or transclusion in a block at point %d, line %d"
+                        (point) (org-current-line))))
    ;; Case 2. #+transclude inside another transclusion
    ((org-transclusion-within-transclusion-p)
-    (user-error "Cannot transclude in another transclusion"))
+    (user-error (format "Cannot transclude in another transclusion at point %d, line %d"
+                        (point) (org-current-line))))
    (t
     t)))
 
@@ -1687,6 +1681,17 @@ When DEMOTE is non-nil, demote."
             (org-transclusion-promote-adjust-after)))
         (goto-char pos)))))
 
+;;-----------------------------------------------------------------------------
+;;;; Functions to support Org-export
+
+(defun org-transclusion-inhibit-read-only (&rest _args)
+  "Set `inhibit-read-only' to t for Org export functions.
+Org export may need the buffer not to contain read-only elements.
+This function is meant to be added to
+`org-export-before-processing-hook' to temporarily inhibit
+read-only."
+  (setq-local inhibit-read-only t))
+
 ;;-----------------------------------------------------------------------------
 ;;;; Functions for extensions
 ;;   It's based on `org-modules'
